/**
 * Created by Samuel Gratzl on 24.11.2014.
 */
import * as tslib_1 from "tslib";
import './_font-awesome';
import './style.scss';
import * as template from 'html-loader!./_header.html';
import { mixin } from 'phovea_core/src/index';
import lazyBootstrap from './_lazyBootstrap';
import buildBuildInfo from './buildInfo';
import getMetaData from './metaData';
/**
 * Header link extends the header link with a  flag for disabling the logo
 */
var AppHeaderLink = /** @class */ (function () {
    function AppHeaderLink(name, action, href, addLogo) {
        if (name === void 0) { name = 'Phovea'; }
        if (action === void 0) { action = function (event) { return false; }; }
        if (href === void 0) { href = '#'; }
        if (addLogo === void 0) { addLogo = true; }
        this.name = name;
        this.action = action;
        this.href = href;
        this.addLogo = addLogo;
    }
    return AppHeaderLink;
}());
export { AppHeaderLink };
/**
 * Helper function to create a list item for the header menus
 * @param name
 * @param action
 * @param href
 * @returns {HTMLElement}
 */
function createLi(name, action, href) {
    if (href === void 0) { href = '#'; }
    var li = document.createElement('li');
    li.innerHTML = "<a href=\"" + href + "\">" + name + "</a>";
    if (action) {
        li.querySelector('a').onclick = action;
    }
    return li;
}
/**
 * The Caleydo App Header provides an app name and customizable menus
 */
var AppHeader = /** @class */ (function () {
    /**
     * Constructor overrides the default options with the given options
     * @param parent
     * @param options
     */
    function AppHeader(parent, options) {
        if (options === void 0) { options = {}; }
        this.parent = parent;
        /**
         * Default options that can be overridden in the constructor
         * @private
         */
        this.options = {
            /**
             * insert as first-child or append as child node to the given parent node
             */
            prepend: true,
            /**
             * color scheme: bright (= false) or dark (= true)
             */
            inverse: false,
            /**
             * @DEPRECATED use `appLink.name` instead
             */
            //app: 'Caleydo Web',
            /**
             * @DEPRECATED use `appLink.addLogo` instead
             */
            //addLogo: true,
            /**
             * the app link with the app name
             */
            appLink: new AppHeaderLink(),
            /**
             * a list of links that should be shown in the main menu
             */
            mainMenu: [],
            /**
             * a list of links that should be shown in the right menu
             */
            rightMenu: [],
            /**
             * show/hide the options link
             */
            showOptionsLink: false,
            /**
             * show/hide the bug report link
             */
            showReportBugLink: true,
            /**
             * show/hide the EU cookie disclaimer bar from `cookie-bar.eu`
             */
            showCookieDisclaimer: false,
            /**
             * show help link
             */
            showHelpLink: false
        };
        mixin(this.options, options);
        this.addEUCookieDisclaimer();
        this.build();
    }
    AppHeader.prototype.addEUCookieDisclaimer = function () {
        if (!this.options.showCookieDisclaimer) {
            return;
        }
        var script = document.createElement('script');
        script.type = 'text/javascript';
        script.async = true;
        script.src = 'https://cdn.jsdelivr.net/npm/cookie-bar/cookiebar-latest.min.js?theme=flying';
        script.onload = function () {
            setupCookieBar();
        };
        this.parent.ownerDocument.body.appendChild(script);
    };
    AppHeader.prototype.build = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            var helper, appLink;
            return tslib_1.__generator(this, function (_a) {
                // legacy support
                if (this.options.app !== undefined && this.options.appLink === undefined) {
                    this.options.appLink.name = this.options.app;
                }
                if (this.options.addLogo !== undefined && !this.options.appLink === undefined) {
                    this.options.appLink.addLogo = this.options.addLogo;
                }
                helper = document.createElement('div');
                helper.innerHTML = String(template);
                while (helper.lastChild) {
                    this.parent.insertBefore(helper.lastChild, this.parent.firstChild);
                }
                // use the inverse color scheme
                if (this.options.inverse) {
                    this.parent.querySelector('nav.navbar').classList.add('navbar-inverse');
                }
                appLink = this.parent.querySelector('*[data-header="appLink"]');
                appLink.innerHTML = this.options.appLink.name;
                appLink.onclick = this.options.appLink.action;
                appLink.setAttribute('href', this.options.appLink.href);
                if (this.options.appLink.addLogo) {
                    appLink.classList.add('caleydo_app');
                }
                this.mainMenu = this.parent.querySelector('*[data-header="mainMenu"]');
                this.rightMenu = this.parent.querySelector('*[data-header="rightMenu"]');
                this.aboutDialog = this.parent.querySelector('*[data-header="about"]');
                this.optionsDialog = this.parent.querySelector('*[data-header="options"]');
                // show/hide links
                this.toggleOptionsLink(this.options.showOptionsLink !== false, typeof this.options.showOptionsLink === 'function' ? this.options.showOptionsLink : null);
                this.toggleAboutLink(this.options.showAboutLink !== false, typeof this.options.showAboutLink === 'function' ? this.options.showAboutLink : null);
                this.toggleReportBugLink(this.options.showReportBugLink !== false, typeof this.options.showReportBugLink === 'function' ? this.options.showReportBugLink : null);
                this.toggleHelpLink(this.options.showHelpLink !== false, typeof this.options.showHelpLink === 'string' ? this.options.showHelpLink : null);
                this.options.mainMenu.forEach(function (l) { return _this.addMainMenu(l.name, l.action, l.href); });
                this.options.rightMenu.forEach(function (l) { return _this.addRightMenu(l.name, l.action, l.href); });
                return [2 /*return*/];
            });
        });
    };
    AppHeader.prototype.addMainMenu = function (name, action, href) {
        if (href === void 0) { href = '#'; }
        var li = createLi(name, action, href);
        this.mainMenu.appendChild(li);
        return li;
    };
    AppHeader.prototype.addRightMenu = function (name, action, href) {
        if (href === void 0) { href = '#'; }
        var li = createLi(name, action, href);
        this.rightMenu.insertBefore(li, this.rightMenu.firstChild);
        return li;
    };
    AppHeader.prototype.insertCustomMenu = function (element) {
        this.rightMenu.parentElement.insertBefore(element, this.rightMenu);
    };
    AppHeader.prototype.insertCustomRightMenu = function (element) {
        this.rightMenu.parentElement.appendChild(element);
    };
    AppHeader.prototype.wait = function () {
        AppHeader.setVisibility(document.querySelector('#headerWaitingOverlay'), true);
    };
    AppHeader.prototype.ready = function () {
        AppHeader.setVisibility(document.querySelector('#headerWaitingOverlay'), false);
    };
    AppHeader.setVisibility = function (element, isVisible) {
        element.classList.toggle('hidden', !isVisible);
    };
    AppHeader.prototype.toggleOptionsLink = function (isVisible, contentGenerator) {
        var _this = this;
        var link = this.parent.querySelector('*[data-header="optionsLink"]');
        AppHeader.setVisibility(link, isVisible);
        // set the URL to GitHub issues dynamically
        if (isVisible) {
            contentGenerator = contentGenerator || defaultOptionsInfo;
            lazyBootstrap().then(function ($) {
                $('#headerOptionsDialog').one('show.bs.modal', function () {
                    var content = _this.parent.querySelector('*[data-header="options"]');
                    var title = _this.parent.querySelector('#headerOptionsDialog .modal-title');
                    content.innerHTML = 'Loading...';
                    contentGenerator(title, content);
                });
            });
        }
    };
    AppHeader.prototype.toggleHelpLink = function (isVisible, helpUrl) {
        var link = this.parent.querySelector('*[data-header="helpLink"]');
        AppHeader.setVisibility(link, isVisible);
        if (isVisible && helpUrl) {
            link.querySelector('a').href = helpUrl;
        }
    };
    AppHeader.prototype.toggleReportBugLink = function (isVisible, contentGenerator) {
        var _this = this;
        var link = this.parent.querySelector('*[data-header="bugLink"]');
        AppHeader.setVisibility(link, isVisible);
        // set the URL to GitHub issues dynamically
        if (isVisible) {
            contentGenerator = contentGenerator || defaultBuildInfo;
            lazyBootstrap().then(function ($) {
                $('#headerReportBugDialog').one('show.bs.modal', function () {
                    var content = _this.parent.querySelector('*[data-header="bug"]');
                    var title = _this.parent.querySelector('#headerReportBugDialog .modal-title');
                    content.innerHTML = 'Loading...';
                    contentGenerator(title, content);
                });
            });
        }
    };
    AppHeader.prototype.toggleAboutLink = function (isVisible, contentGenerator) {
        var _this = this;
        var link = this.parent.querySelector('*[data-header="aboutLink"]');
        AppHeader.setVisibility(link, isVisible);
        if (isVisible) {
            contentGenerator = contentGenerator || defaultAboutInfo;
            var modifyDialogOnce_1 = function () {
                // request last deployment data
                var content = _this.aboutDialog;
                var title = _this.aboutDialog.parentElement.querySelector('.modal-title');
                contentGenerator(title, content);
                // remove event listener to prevent another DOM modification
                link.removeEventListener('click', modifyDialogOnce_1);
            };
            link.addEventListener('click', modifyDialogOnce_1);
        }
    };
    AppHeader.prototype.hideDialog = function (selector) {
        lazyBootstrap().then(function ($) {
            $(selector).modal('hide');
        });
    };
    AppHeader.prototype.showAndFocusOn = function (selector, focusSelector) {
        lazyBootstrap().then(function ($) {
            $(selector).modal('show')
                .on('shown.bs.modal', function () {
                $(focusSelector, selector).focus();
            });
        });
    };
    return AppHeader;
}());
export { AppHeader };
export function create(parent, options) {
    if (options === void 0) { options = {}; }
    return new AppHeader(parent, options);
}
function defaultBuildInfo(_title, content) {
    buildBuildInfo().then(function (buildInfo) {
        content.innerHTML = buildInfo.toHTML();
    }).catch(function (error) {
        content.innerHTML = error.toString();
    });
}
function defaultAboutInfo(title, content) {
    content = content.querySelector('.metaData');
    getMetaData().then(function (metaData) {
        title.innerHTML = (metaData.displayName || metaData.name).replace('_', ' ');
        var contentTpl = "<p class=\"description\">" + metaData.description + "</p>";
        if (metaData.homepage) {
            contentTpl += "<p class=\"homepage\"><strong>Homepage</strong>: <a href=\"" + metaData.homepage + "\" target=\"_blank\" rel=\"noopener\">" + metaData.homepage + "</a></p>";
        }
        contentTpl += "<p class=\"version\"><strong>Version</strong>: " + metaData.version + "</p>";
        if (metaData.screenshot) {
            contentTpl += "<img src=\"" + metaData.screenshot + "\" class=\"center-block img-responsive img-thumbnail\"/>";
        }
        content.innerHTML = contentTpl;
    });
}
function defaultOptionsInfo(_title, content) {
    content.innerHTML = 'No options available';
}
//# sourceMappingURL=header.js.map